/*
   pid.c -- PIKT routines to find pid's and ppid's

   $Header: /home/cvs/ucgsb/pikt/src/lib/pid.c,v 1.6 2001/01/12 14:39:43 berto Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998-2001 by Robert Osterlund.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   Send problem reports, bouquets and brickbats to:

   robert.osterlund@gsb.uchicago.edu

   Robert Osterlund
   Graduate School of Business
   University of Chicago
   1101 E. 58th Street, W309
   Chicago, IL 60637
   USA

   --------------------------------------------------------------------------

   I strongly suspect that there are better ways to do this!
   Suggestions, anyone?  Help!

*/

#include "common.h"

/* for the given process, return either its PID or PPID

   we used to invoke ioctl() on each /proc/<pid> to get the process names,
   but we saw several cases where the resultant process name for a given
   pid didn't match the process name for that pid as shown in the ps display;
   the technique of reading the ps display, although perhaps less elegant,
   gives us the consistent results we desire
 */
pid_t
procid (char *procname, int type)
{
	static char fnnam[] = "procid";
	pid_t pspid;
	long pid, ppid;
	int procnamelen;
	#if PSPARSE == 1
	char psp[MAXLINE+1];
	#endif
	char *psproc;
	char line[MAXLINE+1];
	FILE *ps;
	char *cp;

	pspid = 0;
	procnamelen = strlen(procname);
	if ((ps = popen(PSCMD " " PSOPTS, "r")) == NULL) {
		logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in %s(), popen() error", fnnam);
		return ERROR;
	}
	fgets(line, MAXLINE+1, ps);	/* throw away header */
	while (fgets(line, MAXLINE+1, ps)) {
		*strchr(line, '\n') = '\0';
		#if PSPARSE == 1
		if (sscanf(line, "%ld %ld %s", &pid, &ppid, psp) != 3) {
			logreturn(PIKTLOG, LOG_WARNING, __FILE__, __LINE__, FALSE, "in %s(), error reading pid/ppid/psproc for line %s", fnnam, line);
			continue;
		}
		/* strip away path, if any, leaving only basename */
		if ((cp = strrchr(psp, '/')))
			psproc = cp+1;
		else
			psproc = psp;
		#else
		# if PSPARSE == 2
		if (sscanf(line, "%*s %*s %ld %ld %*s %*s %*s %*s %*s %*s %*s %*s %*s", &pid, &ppid) != 2) {
			logreturn(PIKTLOG, LOG_WARNING, __FILE__, __LINE__, FALSE, "in %s(), error reading pid/ppid for line %s", fnnam, line);
			continue;
		}
		if ((psproc = strrchr(line, ':')))
			psproc += 4;
		else {
			logreturn(PIKTLOG, LOG_WARNING, __FILE__, __LINE__, FALSE, "in %s(), error reading psproc for line %s", fnnam, line);
			continue;
		}
		# else
		#  if PSPARSE == 3
		if (sscanf(line, "%*s %*s %*s %ld %ld %*s %*s %*s %*s %*s %*s %*s %*s %*s", &pid, &ppid) != 2) {
			logreturn(PIKTLOG, LOG_WARNING, __FILE__, __LINE__, FALSE, "in %s(), error reading pid/ppid for line %s", fnnam, line);
			continue;
		}
		if ((psproc = strrchr(line, ':')))
			psproc += 4;
		else {
			logreturn(PIKTLOG, LOG_WARNING, __FILE__, __LINE__, FALSE, "in %s(), error reading psproc for line %s", fnnam, line);
			continue;
		}
		#  else
		#   include "Bletch: How do we parse `ps' output?"
		#  endif
		# endif
		#endif
		/* for some strange reason, linux ps appends a space to the
		   end of each ps line
		 */
		if (! strncmp(psproc, procname, (size_t)procnamelen) && ((*(psproc+procnamelen) == '\0') || (*(psproc+procnamelen) == ' '))) {
			if (strcmp(procname, "piktd") || (ppid == 1)) {
				if (pspid == 0) {
					switch (type) {
						case PROCID:
							pspid = (pid_t) pid;
							break;
						case PPROCID:
							pspid = (pid_t) ppid;
							break;
					}
				} else {
					logreturn(PIKTLOG, strcmp(procname, "piktd") ? LOG_WARNING : LOG_ERR, __FILE__, __LINE__, FALSE, "in %s(), failed to determine unique pid for proc %s: multiple processes running", fnnam, procname);
					pclose(ps);
					return ERROR;
				}
			}
		}
	}
	pclose(ps);
	if ((pspid == 0)
		#if defined(SKIP_PROC)
		&& strcmp(procname, SKIP_PROC)
		#endif
						) {
		if (! strcmp(procname, "piktd"))
			logreturn(PIKTLOG, LOG_WARNING, __FILE__, __LINE__, FALSE, "in %s(), failed to determine pid for proc %s: process not running", fnnam, procname);
		return NIL;
	} else
		return pspid;
}
