%{

/*
   cfg.l -- PIKT piktc lexer for all .cfg files except systems.cfg

   $Header: /home/cvs/ucgsb/pikt/src/piktc/cfg.l,v 1.60 2001/04/20 18:46:23 berto Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998-2001 by Robert Osterlund.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   Send problem reports, bouquets and brickbats to:

   robert.osterlund@gsb.uchicago.edu

   Robert Osterlund
   Graduate School of Business
   University of Chicago
   1101 E. 58th Street, W309
   Chicago, IL 60637
   USA

   ----------------------------------------------------------------------------

   this is the sausage factory; not for the squeamish; enter at your own risk!

   on our test irix system, flex failed to match regexps of the form
   '{D}{x,y}', which led to parser failures; hence substitutions of the
   form '{D}+' below

 */

#include "../lib/common.h"
#include "piktc.h"
#include "cfg.tab.h"

char _cfglex_[] = "cfglex";

#define RETURN(token) cfglval.sval = mystrdup(yytext, _cfglex_); return token

#undef YY_INPUT
#define YY_INPUT(buf, result, max_size) { \
		signed char c = cfginput(); \
		result = (c == EOF) ? YY_NULL : (buf[0] = c, 1); \
	}

int ii, jj;
int fn = 0;

int context;	/* STRING/NUMBER */

void cfgerror(const char *);
signed int cfginput(void);

%}

W		[ \t]
WN		[ \t\n]
D		[0-9]
L		[A-Za-z]
DL		[0-9A-Za-z]
DLU		[0-9A-Za-z_]

%x MDS MPS AS ASE PS FS OS CS DS SS FNS LS CMNT2

%option noyywrap

%%

"/"{3,8}\n			{ ii = 0;
				  switch(strlen(yytext)-1) {
					case 3:
						return _MCRCFGBOF;
						break;
					case 4:
						return _ALMCFGBOF;
						break;
					case 5:
						return _ALTCFGBOF;
						break;
					case 6:
						return _PRGCFGBOF;
						break;
					case 7:
						return _FILCFGBOF;
						break;
					case 8:
						return _OBJCFGBOF;
						break;
				  }
				}

<MDS,AS,LS,OS>"/"{3,8}\n	{ yyless(0);
				  BEGIN(INITIAL);
				}

<MDS,AS,LS,OS>\177"/"+\n	{ /* <MDS,AS,LS,OS>\177"/"{3,8}\n */
				  yyless(1);
				  BEGIN(INITIAL);
				}

<*>^{D}+{W}*\n			{ lineno = atoi(yytext); }

<*>\177				{ ; /* dummy char; see
				       cfginput() below
				     */
				}

<*>^({L}|"/")[-_\./0-9A-Za-z]*	{ if ((yytext[0] == '/') && (cfgcurr != PRGCFG) && (cfgcurr != FILCFG))
					cfgerror("syntax error");
				  if (yyleng > MAXIDENT) {
					logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "stanza name length exceeds MAXIDENT: %s", yytext);
					bailout = TRUE;
					*(yytext+MAXIDENT) = '\0';
				  }
				  switch(cfgcurr) {
				  	case MCRCFG:
						BEGIN(MDS);
						break;
					case ALTCFG:
						context = NIL;
						BEGIN(AS);
						break;
					case PRGCFG:
						ii = 0;
						BEGIN(PS);
						break;
					case FILCFG:
						ii = 0;
						BEGIN(FS);
						break;
					case OBJCFG:
						BEGIN(OS);
						break;
					default:
					  	BEGIN(INITIAL);
						break;
				  }
				  RETURN(_STANZA);
				}

<INITIAL,MDS>^{L}[-_\.0-9A-Za-z]*/"("[^ \t\n]+ {
				  if (cfgcurr == MCRCFG) {
					if (yyleng > MAXIDENT) {
						logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "stanza name length exceeds MAXIDENT: %s", yytext);
						bailout = TRUE;
						*(yytext+MAXIDENT) = '\0';
					}
					BEGIN(MPS);
					RETURN(_STANZA);
				  } else
					cfgerror("syntax error");
				}

<MDS>=(piktnullchar|_)		{ cfglval.sval = mystrdup(piktnullchar, _cfglex_);
				  return _MACRODEF;
				}
<MDS>=piktspacechar		{ cfglval.sval = mystrdup(piktspacechar, _cfglex_);
				  return _MACRODEF;
				}
<MDS>=pikttabchar		{ cfglval.sval = mystrdup(pikttabchar, _cfglex_);
				  return _MACRODEF;
				}
<MDS>=piktnewlinechar		{ cfglval.sval = mystrdup(piktnewlinechar, _cfglex_);
				  return _MACRODEF;
				}
<MDS>[^ \t\n]+			{ RETURN(_MACRODEF); }
<MDS>\"[^\"]*\"			{ RETURN(_MACRODEF); }

<MPS>"("			{ return yytext[0]; }
<MPS>")"			{ BEGIN(MDS);
				  return yytext[0];
				}
<MPS>","			{ return yytext[0]; }
<MPS>{L}{DLU}*			{ RETURN(_MACROPRM); }

<INITIAL,DS,SS>status		{ RETURN(_STATUS); }
<INITIAL,DS,SS>active		{ RETURN(_ACTIVE); }
<INITIAL,DS,SS>inactive		{ RETURN(_INACTIVE); }

<INITIAL,DS,SS>level		{ RETURN(_LEVEL); }
<INITIAL,DS,SS>emerg|emergency	{ RETURN(_EMERG); }
<INITIAL,DS,SS>alert|urgent	{ RETURN(_ALERT); }
<INITIAL,DS,SS>crit|critical	{ RETURN(_CRIT); }
<INITIAL,DS,SS>err|error	{ RETURN(_ERR); }
<INITIAL,DS,SS>warn|warning	{ RETURN(_WARNING); }
<INITIAL,DS,SS>notice		{ RETURN(_NOTICE); }
<INITIAL,DS,SS>info		{ RETURN(_INFO); }
<INITIAL,DS,SS>debug		{ RETURN(_DEBUG); }

<INITIAL,DS,SS>task		{ RETURN(_TASK); }

<INITIAL,DS,SS>input		{ RETURN(_INPUT); }
<INITIAL,DS,SS>proc|process	{ BEGIN(CS);
				  RETURN(_PROCESS);
				}
<INITIAL,DS,SS>file		{ RETURN(_REGFILE); }
<INITIAL,DS,SS>log|logfile	{ RETURN(_LOGFILE); }

<INITIAL,DS,SS>filter		{ BEGIN(CS);
				  RETURN(_FILTER);
				}

<INITIAL,DS,SS>seps|separators	{ RETURN(_SEPARATORS); }
<INITIAL,DS,SS>dat|data		{ BEGIN(DS);
				  RETURN(_DAT);
				}
<INITIAL,DS,SS>keys		{ BEGIN(DS);
				  RETURN(_KEYS);
				}

<INITIAL,DS,SS>init		{ RETURN(_INIT); }
<INITIAL,DS,SS,CS>begin		{ BEGIN(INITIAL);
				  RETURN(_BEGIN);
				}
<INITIAL,DS,SS,CS>end		{ BEGIN(INITIAL);
				  RETURN(_END);
				}
<INITIAL,DS,SS,CS>rule		{ BEGIN(INITIAL);
				  RETURN(_RULE);
				}

<INITIAL,DS,SS>do		{ context = NIL;
				  RETURN(_DO);
				}
<INITIAL,DS,SS>if		{ context = NUMBER;
				  RETURN(_IF);
				}
<INITIAL,DS,SS>if{WN}*"("	{ context = NUMBER;
				  unput('[');
				  cfglval.sval = mystrdup("if", _cfglex_);
				  return _IF;
				}
<INITIAL,DS,SS>(elif|elsif|elseif) {
				  context = NUMBER;
				  RETURN(_ELIF);
				}
<INITIAL,DS,SS>(elif|elsif|elseif){WN}*"(" {
				  context = NUMBER;
				  unput('[');
				  cfglval.sval = mystrdup("elif", _cfglex_);
				  return _ELIF;
				}
<INITIAL,DS,SS>else		{ context = NIL;
				  RETURN(_ELSE);
				}
<INITIAL,DS,SS>endif|fi		{ context = NIL;
				  RETURN(_ENDIF);
				}
<INITIAL,DS,SS>while		{ context = NUMBER;
				  RETURN(_WHILE);
				}
<INITIAL,DS,SS>while{WN}*"("	{ context = NUMBER;
				  unput('[');
				  cfglval.sval = mystrdup("while", _cfglex_);
				  return _WHILE;
				}
<INITIAL,DS,SS>endwhile		{ context = NIL;
				  RETURN(_ENDWHILE); }
<INITIAL,DS,SS>repeat		{ context = NIL;
				  RETURN(_REPEAT);
				}
<INITIAL,DS,SS>until		{ context = NUMBER;
				  RETURN(_UNTIL); }
<INITIAL,DS,SS>until{WN}*"("	{ context = NUMBER;
				  unput('[');
				  cfglval.sval = mystrdup("until", _cfglex_);
				  return _UNTIL;
				}
<INITIAL,DS,SS>for		{ context = NUMBER;
				  RETURN(_FOR);
				}
<INITIAL,DS,SS>for{WN}*"("	{ context = NUMBER;
				  RETURN(_FORLP);
				}
<INITIAL,DS,SS>foreach		{ context = STRING;
				  RETURN(_FOREACH);
				}
<INITIAL,DS,SS>endfor|endforeach {
				  context = NIL;
				  RETURN(_ENDFOR);
				}
<INITIAL,DS,SS>in		{ context = STRING;
				  RETURN(_IN);
				}
<INITIAL,DS,SS>break		{ context = NIL;
				  RETURN(_BREAK);
				}
<INITIAL,DS,SS>continue|cont	{ context = NIL;
				  RETURN(_CONTINUE);
				}
<INITIAL,DS,SS>switch		{ context = NIL;
				  RETURN(_SWITCH);
				}
<INITIAL,DS,SS>switch{WN}*"("	{ context = NIL;
				  RETURN(_SWITCHLP);
				}
<INITIAL,DS,SS>case		{ context = NIL;
				  RETURN(_CASE);
				}
<INITIAL,DS,SS>default		{ context = NIL;
				  RETURN(_DEFAULT);
				}
<INITIAL,DS,SS>breakswitch|breaksw {
				  context = NIL;
				  RETURN(_BREAKSWITCH);
				}
<INITIAL,DS,SS>endswitch|endsw	{ context = NIL;
				  RETURN(_ENDSWITCH);
				}
<INITIAL,DS,SS>again		{ context = NIL;
				  RETURN(_AGAIN);
				}
<INITIAL,DS,SS>leave		{ context = NIL;
				  RETURN(_LEAVE);
				}
<INITIAL,DS,SS>redo		{ context = NIL;
				  RETURN(_REDO);
				}
<INITIAL,DS,SS>next		{ context = NIL;
				  RETURN(_NEXT);
				}
<INITIAL,DS,SS>skip		{ context = NUMBER;
				  RETURN(_SKIP);
				}
<INITIAL,DS,SS>last		{ context = NIL;
				  RETURN(_LAST);
				}
<INITIAL,DS,SS>pause		{ context = NUMBER;
				  RETURN(_PAUSE);
				}
<INITIAL,DS,SS>quit		{ context = NIL;
				  RETURN(_QUIT);
				}
<INITIAL,DS,SS>die		{ context = STRING;
				  RETURN(_DIE);
				}

<INITIAL,DS,SS>output		{ context = STRING;
				  RETURN(_OUTPUT);
				}
<INITIAL,DS,SS>mail		{ RETURN(_MAIL); }
<INITIAL,DS,SS>print		{ RETURN(_PRINT); }
<INITIAL,DS,SS>piktlog		{ RETURN(_PIKTLOG); }
<INITIAL,DS,SS>syslog		{ RETURN(_SYSLOG); }

<INITIAL,DS,SS>set		{ context = NIL;
				  BEGIN(SS);
				  RETURN(_SET);
				}
<INITIAL,DS,SS>unset|delete	{ context = NIL;
				  BEGIN(SS);
				  RETURN(_UNSET);
				}

<INITIAL,DS,SS>exec		{ context = STRING;
				  BEGIN(INITIAL);
				  RETURN(_EXEC);
				}
<INITIAL>wait			{ RETURN(_WAIT); }

<CS,DS>\"[^\n]+\"/{W}*\n	{ BEGIN(INITIAL);
				  RETURN(_STR);
				}
<DS>\"[^\"]+\"			{ RETURN(_STR); }

<DS>({D}+|$|$-{D}+)/[^\]0-9A-Za-z_] {
				  /* <DS>({D}{1,5}|$|$-{D}{1,5})/[^\]0-9A-Za-z_] */
				  BEGIN(INITIAL);
				  RETURN(_POSITION);
				}
<DS>"["({D}+|$|$-{D}+)"]" 	{ /* <DS>"["({D}{1,5}|$|$-{D}{1,5})"]" */
				  BEGIN(INITIAL);
				  RETURN(_POSITION);
				}
<DS>{D}+","({D}+|$)		{ BEGIN(INITIAL);
				  RETURN(_RANGE);
				}
<DS>"["{D}+","({D}+|$)"]"	{ BEGIN(INITIAL);
				  RETURN(_RANGE);
				}

<DS>(#|$){L}{DLU}*"[" 		{ yytext[yyleng-1] = '\0';
				  cfglval.sval = mystrdup(yytext, _cfglex_);
				  unput('[');
				  unput('[');
				  unput('[');
				  /*
				  context = (cfglval.sval[0] == '#' ? NUMBER : STRING);
				   */
				  return (cfglval.sval[0] == '#' ? _VARNUM : _VARSTR);
				}

<INITIAL,SS>"="			{ context = NIL;
				  BEGIN(INITIAL);
				  return yytext[0];
				}
<INITIAL,SS>("+="|"-="|"*="|"/="|"%="|"**="|"<<="|">>="|"&="|"|="|"^=") {
				  context = NUMBER;
				  BEGIN(INITIAL);
				  RETURN(_AOPNUM);
				}
<INITIAL,SS>".="		{ context = STRING;
				  BEGIN(INITIAL);
				  RETURN(_AOPSTR);
				}

<FNS>[A-Z][0-9A-Z]*		{ RETURN(_FLHNDL); }

<INITIAL,SS,FNS>${D}+		{ /* <INITIAL,SS,FNS>${D}{1,5} */
				  context = STRING;
				  RETURN(_VALSTR);
				}

<INITIAL,DS,SS,FNS>(#|$){L}{DLU}* { context = (yytext[0] == '#' ? NUMBER : STRING);
				  RETURN(yytext[0] == '#' ? _VARNUM : _VARSTR);
				}

<INITIAL,SS,FNS>$"["		{ yytext[yyleng-1] = '\0';
				  cfglval.sval = mystrdup(yytext, _cfglex_);
				  unput('[');
				  unput('[');
				  unput('[');
				  context = STRING;
				  return _VALSTR;
				}

<INITIAL,SS,FS>(#|$){L}{DLU}*{WN}*"[" {
				  ii = yyleng-2;
				  while (isspace((int)*(yytext+ii)))
					ii--;
				  *(yytext+ii+1) = '\0';
				  cfglval.sval = mystrdup(yytext, _cfglex_);
				  unput('[');
				  unput('[');
				  unput('[');
				  /*
				  context = (cfglval.sval[0] == '#' ? NUMBER : STRING);
				   */
				  return (cfglval.sval[0] == '#' ? _VARNUM : _VARSTR);
				}

<INITIAL,FNS>(%|@){L}{DLU}*/("["[^\n]+"]")*{WN}*("+"|"-"|"*"|"/"|"%"|"**"|"& "|"| "|"^"|"<<"|">>"|"=="|">"|"<"|">="|"<="|"!>"|"!<"|"!="|"<>"|"]"){WN}* {
				  context = NUMBER;
				  RETURN(yytext[0] == '%' ? _VARHSTNUM : _VARPRCNUM);
				}

<INITIAL,FNS>(%|@){L}{DLU}*/("["[^\[\]]+"]")*{WN}*("."|"=~"|"=~~"|"!~"|"!~~"){WN}* {
				  context = STRING;
				  RETURN(yytext[0] == '%' ? _VARHSTSTR : _VARPRCSTR);
				}

<INITIAL,FNS>(%|@){L}{DLU}*/("["[^\[\]]+"]")*{WN}+(eq|ne|lt|gt|le|ge){WN}* {
				  context = STRING;
				  RETURN(yytext[0] == '%' ? _VARHSTSTR : _VARPRCSTR);
				}

<INITIAL,FNS>(%|@){L}{DLU}*	{ switch (context) {
					case NUMBER:
						RETURN(yytext[0] == '%' ? _VARHSTNUM : _VARPRCNUM);
						break;
					case STRING:
					case NIL:
					default:
						RETURN(yytext[0] == '%' ? _VARHSTSTR : _VARPRCSTR);
						break;
				  }
				}

<INITIAL,DS,SS,FS>"["/{WN}*($|\") {
				  context = STRING;
				  return yytext[0];
				}
<INITIAL,DS,SS,FNS>"["		{ context = NUMBER;
				  return yytext[0];
				}
<INITIAL,DS,SS,FNS>"[[["	{ return '['; }
<INITIAL,DS,SS,FNS>{D}+		{ /* <INITIAL,DS,SS,FNS>{D}{1,5} */
				  context = NUMBER;
				  RETURN(_INT);
				}
<INITIAL,DS,SS,FNS>"]"		{ return yytext[0]; }

<INITIAL,SS,FNS>("+"|"-"|"~")	{ context = NUMBER;
				  return yytext[0];
				}

<INITIAL,SS,FNS>("*"|"/"|"%"|"**"|"&"|"|"|"^"|"<<"|">>") {
				  context = NUMBER;
				  RETURN(_NOP);
				}

<INITIAL,SS,FNS>"."		{ context = STRING;
				  RETURN(_SOP);
				}

<INITIAL,SS,FNS>(-e|-z|-f|-d|-c|-b|-p|-l|-S) {
				  context = STRING;
				  RETURN(_FOP);
				}

<INITIAL,SS,FNS>("=="|">"|"<"|">="|"<="|"!>"|"!<"|"!="|"<>") {
				  context = NUMBER;
				  RETURN(_NREL);
				}
<INITIAL,SS,FNS>("=~"|"=~~"|"!~"|"!~~"|eq|ne|lt|gt|le|ge) {
				  context = STRING;
				  RETURN(_SREL);
				}

<INITIAL,SS,FNS>(({D}+)|({D}*\.{D}+)|((({D}+)|({D}*\.{D}+))\/(({D}+)|({D}*\.{D}+))))%* {
				  context = NUMBER;
				  RETURN(_NUM);
				}

<INITIAL,SS,FNS>(#|$){L}{DLU}*/{WN}*"(" {
				  context = (yytext[0] == '#' ? NUMBER : STRING);
				  BEGIN(FNS);
				  RETURN(yytext[0] == '#' ? _FNNUM : _FNSTR);
				}
<SS,FNS>"("			{ context = NIL;
				  fn++;
				  return '(';
				}
<SS,FNS>")"			{ context = NIL;
				  fn--;
				  if (! fn)
					BEGIN(INITIAL);
				  return ')';
				}

<INITIAL,SS,FS,FNS>\"[^\n]*\"	{ context = STRING;
				  jj = leadstr(yytext);
				  for (ii=yyleng-1; ii>=jj; ii--)
					unput(*(yytext+ii));
				  yytext[ii+1] = '\0';
				  RETURN(_STR);
				}

<INITIAL>"("[\n\t ]*[^\n\t ]+[\n\t ]*(=~|=~~|!~|!~~|eq|ne|lt|gt|le|ge)[\n\t ]*[^\n\t ]+[\n\t ]*")"[\n\t ]* { 
				  context = STRING;
				  yyless(1);
				  return '[';
				}

<INITIAL>"("([-+*/%$#@~^><)(,.0-9A-Za-z_\n\t\[\] ]|"&"[^&]|"|"[^|])+ {
				  if (condparen(yytext, context)) {
					context = NUMBER;
					yyless(1);
					return '[';
				  } else {
					context = NUMBER;
					yyless(1);
					return yytext[0];
				  }
				}

<INITIAL>")"				{ return yytext[0]; }

<INITIAL,SS,FNS>"!"		{ context = NUMBER;
				  RETURN(_NOT);
				}
<INITIAL,SS,FNS>"&&"		{ context = NUMBER;
				  RETURN(_AND);
				}
<INITIAL,SS,FNS>"||"		{ context = NUMBER;
				  RETURN(_OR);
				}

<AS>timing			{ return _TIMING; }
<AS>drift			{ return _DRIFT; }
<AS>(priority|nice)		{ return _PRIORITY; }
<AS>nicecmd			{ return _NICECMD; }
<AS>mailcmd			{ return _MAILCMD; }
<AS>lpcmd			{ return _LPCMD; }
<AS>execcmd			{ BEGIN(ASE);
				  return _EXECCMD;
				}
<AS>(alarms|scripts)		{ return _ALARMS; }
<AS>[-,*0-9]+			{ RETURN(_TIMEVAL); }
<AS>{D}+-{D}+			{ /* <AS>{D}{1,2}-{D}{1,2} */
				  RETURN(_TIMEVAL); }
<AS>"*/"{D}+			{ /* <AS>"\*\/"{D}{1,2} */
				  RETURN(_TIMEVAL); }
<AS>{D}+-{D}+"/"{D}+		{ /* <AS>{D}{1,2}-{D}{1,2}"/"{D}{1,2} */
				  RETURN(_TIMEVAL); }
<AS>{D}+%			{ /* <AS>{D}{1,3}% */
				  RETURN(_TIMEVAL); }
<AS>\"[^\"]*\"			{ yytext[yyleng-1] = '\0';
				  cfglval.sval = mystrdup(yytext+1, _cfglex_);
				  return _STR;
				}
<ASE>\"[^\"]*\"			{ BEGIN(AS);
				  RETURN(_STR);
				}
<AS>[^ \t\n]+			{ RETURN(_WORD); }

<PS,FS>[\n\t ]*path[\n\t 0-9]*	{ return _PATH; }
<PS,FS>[\n\t ]*[0-9]*[\n\t ]*mode[\n\t ]* { return _MODE; }
<PS,FS>[\n\t ]*uid[\n\t ]*	{ return _UID; }
<PS,FS>[\n\t ]*gid[\n\t ]*	{ return _GID; }
<PS,FS>{W}*{D}+{W}*		{ /* <PS,FS>{W}*{D}{1,5}{W}* */
				  /* trim leading & trailing spaces */
				  jj = yyleng-1;
				  while (isspace((int)*(yytext+jj))) {
					*(yytext+jj) = '\0';
					jj--;
				  }
				  jj = 0;
				  while (isspace((int)*(yytext+jj)))
					jj++;
				  /* if we've seen the 3rd _INT (for mode,
				     uid & gid), we're done with the filespec,
				     and we move on to the file body
				   */
				  if (++ii >= 3)
					BEGIN(LS);
				  cfglval.sval = mystrdup(yytext+jj, _cfglex_);
				  return _INT;
				}
<PS,FS>\"[^\"]*\"/[\n\t 0-9]*mode {
				  /* 0-9 in the pattern match, since we must
				     allow for the possibility of 'mode ...'
				     appearing in a subsequent line but after
				     an intervening lineno line; see also the
				     _PATH & _MODE pattern matches above
				   */
				  yytext[yyleng-1] = '\0';
				  cfglval.sval = mystrdup(yytext+1, _cfglex_);
				  return _STR;
				}
<PS,FS>.			{ unput(yytext[0]);
				  BEGIN(LS);
				}

<LS>\t\n			{ cfglval.sval = mystrdup("", _cfglex_);
				  return _LINE;
				}
<LS>{W}.+/\n			{ cfglval.sval = mystrdup(yytext+1, _cfglex_);
				  return _LINE;
				}

<OS>[^ \t\n][^\n]*		{ RETURN(_OBJECT); }

<*>","				{ return yytext[0]; }

<*>{W}				{ ; }
<*>\n				{ ; }

<*>.				{ return yytext[0]; }

%%

/* cfg error handler
 */
void
cfgerror (const char *message)
{

	if (stzerr)
		return;
	else {
		stzerr = TRUE;
		cfgerr = TRUE;
		if (cfgline && strcmp(cfgline,cfgbof[cfgcurr]) && (cfgcurr <= cfglast)) {
			fprintf(stderr, "%s, at or near line %d: %s (in cfgparse()): %s\n", cfgfilecurr, lineno, message, yytext ? yytext : "");
		} else {
			fprintf(stderr, "%s, at or near line %d: %s (in cfgparse())\n", cfgfileprev ? cfgfileprev : cfgfilecurr, lineno, message);
		}
		fflush(stderr);
		bypass = TRUE;
		BEGIN(INITIAL);
		return;
	}
}

/* for config files macros.cfg, alarms.cfg, alerts.cfg, programs.cfg,
   files.cfg & objects.cfg (not systems.cfg or defines.cfg), feed their
   contents to the lexer input stream;
 */
signed int
cfginput (void)
{
	static char fnnam[] = "cfginput";
	static cfgln *clp;
	static char *line = (char *) NULL;
	static char *freeline = (char *) NULL;
	static char linenum[8];
	char c;

	if ((cfgcurr > cfglast) /* || (cfgcurr > OBJCFG) */) {
		loffset = 0;
		return EOF;
	}

	c = ' ';	/* junk return value, just in case */

	if (! loffset) {
		if (bof) {
			cfgline = line = (char *) cfgbof[cfgcurr];
			clp = configs[cfgcurr];
			cfgfileprev = cfgfilecurr;
			cfgfilecurr = (char *) cfgfilenames[cfgcurr];
			/* zero this flag; at end of config lines, if this
			   is still zero, we know that no config lines were
			   processed for current host
			 */
			linenum[0] = '\0';
		} else if (bon) {
			if (! clp) {
				if (isverboselog)
					logreturn(PIKTLOG, LOG_WARNING, __FILE__, __LINE__, FALSE, "in %s(), missing %s", fnnam, cfgfilecurr);
				cfgcurr++;
				if ((cfgcurr > cfglast) /* || (cfgcurr > OBJCFG) */) {
					loffset = 0;
					return EOF;
				} else {
					loffset = 0;
					lineno = 1;
					bof = TRUE;
					bon = FALSE;
					bol = FALSE;
					return '\177';	/* dummy char */
				}
			} else {
				while ((! bsmember(clp->bs, hbscurr, hbslongs)) || ! clp->doproc) {
					clp = clp->next;
					if (clp) {
						cfgfileprev = cfgfilecurr;
						cfgfilecurr = clp->file;
					} else {
						if (! linenum[0])
							logreturn(PIKTLOG, LOG_WARNING, __FILE__, __LINE__, FALSE, "in %s(), %s, for host %s, no input!", fnnam, cfgfileprev ? cfgfileprev : cfgfilecurr, hostcurr);
						cfgcurr++;
						if ((cfgcurr > cfglast) /* || (cfgcurr > OBJCFG) */) {
							loffset = 0;
							return EOF;
						} else {
							loffset = 0;
							lineno = 1;
							bof = TRUE;
							bon = FALSE;
							bol = FALSE;
							return '\177';	/* dummy char */
						}
					}
				}
			}
			sprintf(linenum, "%-6d\n", clp->lineno);
			cfgline = line = linenum;
		} else if (bol) {
			if (clp->domcr) {
				freeline = cfgline = line = expand(clp->line);
			} else {
				freeline = (char *) NULL;
				cfgline = line = clp->line;
			}
		}
	}

	c = *(line+loffset++);
	#ifdef DEEPDEBUG
	logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "YYSTATE is %d, YY_AT_BOL() is %d, c is %d %c", YYSTATE, YY_AT_BOL(), c, c);
	#endif

	if ((c == '\n') && ! *(line+loffset)) {
		if (bof) {
			bof = FALSE;
			bon = TRUE;
		} else if (bon) {
			bon = FALSE;
			bol = TRUE;
		} else if (bol) {
			FREE(freeline);
			if (clp->next) {
				bon = TRUE;
				clp = clp->next;
				if (clp) {
					cfgfileprev = cfgfilecurr;
					cfgfilecurr = clp->file;
				}
			} else {
				cfgcurr++;
				bof = TRUE;
				bon = FALSE;
				bol = FALSE;
			}
		}
		loffset = 0;
	}

	return c;

}
