/*
   piktc_svc.c -- PIKT master control program (piktc) service daemon

   $Header: /home/cvs/ucgsb/pikt/src/piktc/piktc_svc.c,v 1.41 2001/03/24 04:48:24 berto Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998-2001 by Robert Osterlund.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   Send problem reports, bouquets and brickbats to:

   robert.osterlund@gsb.uchicago.edu

   Robert Osterlund
   Graduate School of Business
   University of Chicago
   1101 E. 58th Street, W309
   Chicago, IL 60637
   USA

   --------------------------------------------------------------------------

   Usage:  piktc_svc [-GhTV]

           -G              run in debug mode
           -h              show program help
           -T              run in test mode
           -V              show version info

 */

#include "../lib/common.h"
#include "piktc.h"

const char piktc_svc_version[] = "PIKT piktc_svc version " VERSION;

/* command-line options */
/* int ischeck = FALSE; */
int isdebug = FALSE;
/* int isdelete = FALSE; */
/* int isdoublecheck = FALSE; */
/* int iscksum = FALSE; */
/* int isdiff = FALSE; */
/* int isdisable = FALSE; */
/* int isenable = FALSE; */
int isexec = FALSE;
int isexecwait = FALSE;
/* int isexpand_macros = FALSE; */
int ishelp = FALSE;
/* int isinstall = FALSE; */
/* int isupdate_includes = FALSE; */
/* int iskill_piktd = FALSE; */
/* int iskill_piktc_svc = FALSE; */
/* int islist_files = FALSE; */
/* int islist_items = FALSE; */
/* int isrestart_piktd = FALSE; */
/* int isrestart_piktc_svc = FALSE; */
/* int isshow = FALSE; */
int isshow_version = FALSE;
int istest = FALSE;
/* int isverbose = FALSE; */

int bailout = FALSE;	/* signifies command-line error */

int lineno = 0;

const signed char err[]    = { ERROR, '\0' };
const signed char nil[]    = { NIL, '\0' };
const char errstr[] = "<<ERR>>";
const char nilstr[] = "<<NIL>>";

/* const signed char dquote[] = { DQTPROXY, '\0' }; */
/* const signed char squote[] = { SQTPROXY, '\0' }; */
/* const char space[]  = " "; */

char *hostserv;
char *rqstserv;
int statserv;

unsigned long services = ALL_SERVICES;	/* all services enabled by default */

unsigned int fork_delay = 1;

int
main (int argc, char *argv[])
{
	static char fnnam[] = "main";
	pid_t pid;
	static char pidstr[8];

	init(argc, argv, (void (*)(void))piktc_svc_usage, piktc_svc_version);
	if (bailout)
		exit(1);

	if (! isfile(FILECMD, TRUE) || ! isfile(PSCMD, TRUE)) {
		/* error(s) logged in PIKTLOG */
		exit(1);
	}

	/* check for blocking lockfile */
	if (islock(piktc_svclock)) {
		fprintf(stderr, "%s found!  Another piktc_svc may be running.  Try killing the running piktc_svc (else rm the %s lockfile).\n", piktc_svclock, piktc_svclock);
		fflush(stderr);
		exit(1);
	}
	/* check for running piktc_svc */
	if (procid("piktc_svc", PROCID) == ERROR) {
		fprintf(stderr, "Another piktc_svc may be running.  Try killing the running piktc_svc (and/or rm the %s lockfile, if any).\n", piktc_svclock);
		fflush(stderr);
		exit(1);
	}
	/* apply the lock, to prevent other concurrent piktc_svc runs */
	pid = getpid();
	/* we add one to the pid, because below when we call
	   piktc_svc_main(), the current process forks a child,
	   which will *probably* (although not necessarily) have
	   the next highest pid
	 */
	/* add three to the pid, because empirically that seems to give the
	   best results (on solaris, at least)
	 */
	sprintf(pidstr, "%u", (unsigned)pid+3);
	if (mklock(piktc_svclock, pidstr, TRUE, TRUE) == ERROR)
		logexit(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in %s(), mklock() error for %s", fnnam, piktc_svclock);

	logreturn(PIKTLOG, LOG_INFO, __FILE__, __LINE__, FALSE, "daemon (re)start");

	/* more initializations:  no initial service registrations */
	hostserv = mystrdup("", fnnam);
	rqstserv = mystrdup("", fnnam);

	piktc_svc_main();	/* go into daemon mode */

	exit(0);	/* never reached */
}

/* output program usage to screen
 */
void
piktc_svc_usage (void)
{
	printf("\nUsage:  piktc_svc [-GhTV]\n\n");
	printf("\t-G\t\t\trun in debug mode\n");
	printf("\t-h\t\t\tshow program help\n");
	printf("\t-T\t\t\trun in test mode\n");
	printf("\t-V\t\t\tshow version info\n\n");
	return;
}
