%{

/*
   tag.y -- PIKT piktc tag parser:  deals with machine conditionals and the
            tagging of config file lines; tagging does not apply to
            systems.cfg!

   $Header: /home/cvs/ucgsb/pikt/src/piktc/tag.y,v 1.22 2001/02/09 23:50:40 berto Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998-2001 by Robert Osterlund.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   Send problem reports, bouquets and brickbats to:

   robert.osterlund@gsb.uchicago.edu

   Robert Osterlund
   Graduate School of Business
   University of Chicago
   1101 E. 58th Street, W309
   Chicago, IL 60637
   USA
 */

#include "../lib/common.h"
#include "piktc.h"

char _tagparse_[] = "tagparse";

#define CHKIFNUM	if (ifnum > MAXIFS) { \
				logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in %s(), %s, line %d, MAXIFS (%d) exceeded", _tagparse_, cfgfilecurr, lineno-1, MAXIFS); \
				bailout = TRUE; \
			} else if (ifnum < 0) { \
				logexit(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in %s(), %s, line %d, ifnum < 0", _tagparse_, cfgfilecurr, lineno-1); \
				bailout = TRUE; \
			}

#define SETBS(a,b)	tbs1 = a; \
			a = b; \
			FREEVOID(tbs1)

int ifnum;

unsigned long *hbscur[MAXIFS+1], *hbscum[MAXIFS+1];
unsigned long *tbs1, *tbs2, *tbs3;

%}

%union {
	char *sval;
	int ival;
	unsigned long *hbsval;
}

%token <sval> _IDENT _LINE _DEF
%token _IF _ELIF _ELSE _ENDIF
%type <hbsval> hset

%left '|'
%left '&'
%nonassoc '!'

%%

start:		{
			if (scanpass == 2) {
				ifnum = 0;
				hbscur[ifnum] = (unsigned long *) mycalloc(hbslongs, sizeof(unsigned long), _tagparse_);
				hbscum[ifnum] = (unsigned long *) mycalloc(hbslongs, sizeof(unsigned long), _tagparse_);
				fill(hbscur[ifnum], hbslongs);
			}
		} lines {
			if (scanpass == 2) {
				FREEVOID(hbscur[ifnum]);
				FREEVOID(hbscum[ifnum]);
			}
		}
		
		;

lines:		line
		| lines line
		;

line:		_LINE {
			if (scanpass == 1) {
				if (cfgcurr == DEFCFG)
					defnum++;
			} else if (scanpass == 2) {
				#ifdef DEEPDEBUG
				if (isdebug)
					logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "%s", $1);
				#endif
				tag(lineno-1, cfgfilecurr, hbscur[ifnum], hbslongs, hasmacro ? domacros : FALSE, TRUE, $1);
			}
		}
		| _DEF {
			if (scanpass == 2)
				tag(lineno-1, cfgfilecurr, hbscur[ifnum], hbslongs, hasmacro ? domacros : FALSE, FALSE, $1);
		}
		| conditional
		;

conditional:	if_ _ENDIF {
			if (scanpass == 2) {
				FREEVOID(hbscur[ifnum]);
				FREEVOID(hbscum[ifnum]);
				ifnum--;
				CHKIFNUM;
			}
		}
		| if_ else_ _ENDIF {
			if (scanpass == 2) {
				FREEVOID(hbscur[ifnum]);
				FREEVOID(hbscum[ifnum]);
				ifnum--;
				CHKIFNUM;
			}
		}
		| if_ elifs_ _ENDIF {
			if (scanpass == 2) {
				FREEVOID(hbscur[ifnum]);
				FREEVOID(hbscum[ifnum]);
				ifnum--;
				CHKIFNUM;
			}
		}
		| if_ elifs_ else_ _ENDIF {
			if (scanpass == 2) {
				FREEVOID(hbscur[ifnum]);
				FREEVOID(hbscum[ifnum]);
				ifnum--;
				CHKIFNUM;
			}
		}
		;

if_:		if
		| if lines
		;

if:		_IF hset {
			if (scanpass == 2) {
				ifnum++;
				CHKIFNUM;
				hbscur[ifnum] = bsintersection($2, hbscur[ifnum-1], hbslongs);
				hbscum[ifnum] = bsintersection($2, hbscur[ifnum-1], hbslongs);
				free($2);
			}
		}
		;

elifs_:		elif
		| elif lines
		| elifs_ elif
		| elifs_ elif lines
		;

elif:		_ELIF hset {
			if (scanpass == 2) {
				tbs2 = bscomplement(hbscum[ifnum], hbslongs);
				tbs3 = bsintersection($2, tbs2, hbslongs);
				free(tbs2);
				SETBS(hbscur[ifnum], bsintersection(tbs3, hbscur[ifnum-1], hbslongs));
				free(tbs3);
				tbs2 = bsunion($2, hbscum[ifnum], hbslongs);
				SETBS(hbscum[ifnum], bsintersection(tbs2, hbscur[ifnum-1], hbslongs));
				free(tbs2);
				free($2);
			}
		}
		;

else_:		else
		| else lines
		;

else:		_ELSE {
			if (scanpass == 2) {
				tbs2 = bscomplement(hbscum[ifnum], hbslongs);
				SETBS(hbscur[ifnum], bsintersection(tbs2, hbscur[ifnum-1], hbslongs));
				free(tbs2);
			}
		}
		;

hset:		_IDENT {
			if (scanpass == 2) {
				/* HST is a stand-in for OPS, GRP & ALS */
				$$ = bsdup(getbs($1, HST, TRUE), hbslongs);
				free($1);
			}
		}
		| '(' hset ')' {
			if (scanpass == 2) {
				$$ = $2;
				/* or :
				$$ = bsdup($2, hbslongs);
				free($2);
				 */
			}
		}
		| '!' hset {
			if (scanpass == 2) {
				$$ = bscomplement($2, hbslongs);
				free($2);
			}
		}
		| hset '&' hset {
			if (scanpass == 2) {
				$$ = bsintersection($1, $3, hbslongs);
				free($1);
				free($3);
			}
		}
		| hset '|' hset {
			if (scanpass == 2) {
				$$ = bsunion($1, $3, hbslongs);
				free($1);
				free($3);
			}
		}
		;

%%
